package com.ibm.jps.uxtoweb.frontend.custom;

import inputSchema.Bean;
import inputSchema.InputSchemaFactory;
import inputSchema.MappedTo;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Dependency;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.Namespace;
import org.eclipse.uml2.uml.Property;

import com.ibm.xtools.transform.authoring.RuleExtension;

public class Class2BeanRule implements RuleExtension {

	@SuppressWarnings("unchecked")
	public void execute(EObject arg0, EObject arg1) {
		ModelUtil.log("Class2BeanRule", arg0);
		Class cl = (Class) arg0;
		Bean bean = (Bean) arg1;
		// Bean method: not used
		// Bean package
		Element owner = cl.getOwner();
		if (owner instanceof NamedElement) {
			bean.setPackage(ModelUtil.convertToFullJavaName(((NamedElement) owner).getQualifiedName()));
		} else {
			bean.setPackage(Messages.unknownPackage);
		}
		// Bean name
		bean.setName(cl.getName());
		// Bean hasPasswordAttribute property
		bean.setHasPasswordAttribute(Messages.falseMark);
		for (Object o:cl.getAllAttributes()) {
			Property prop = (Property) o;
			if (prop.getType() != null && prop.getName().equals(Messages.password)) {
				bean.setHasPasswordAttribute(Messages.trueMark);
				break;
			}
		}
		// Beam mappedTo property
		boolean done = false;
		for (Object o:cl.getAllAttributes()) {
			Property prop = (Property) o;
			// This code cycles through the dependencies of the current attribute. If the supplier of 
			// a dependency is a class stereotyped <<table>>, add a new MappedTo entry to the bean.
			// NOTE: In this version, we stop at the first entry (one bean can be mapped to only one
			// table.
			for (Object o1:prop.getClientDependencies()) {
				Dependency dep = (Dependency) o1;
				for (Object o2:dep.getSuppliers()) {
					NamedElement elt = (NamedElement) o2;
					if (elt instanceof Property) {
						Property att = (Property) elt;
						Namespace ns = att.getNamespace();
						if (ns != null && ns instanceof Class) {
							Class supplier = (Class) ns;
							if (supplier.getAppliedStereotype(Messages.tableStereotype) != null) {
								MappedTo map = InputSchemaFactory.eINSTANCE.createMappedTo();
								bean.getMappedTo().add(map);
								map.setTableName(supplier.getName());
								done = true;
								break;
							}
						}
					}
				}
				if (done) break;
			}
		}
	}
}
