package com.ibm.jps.uxtoweb.frontend.custom;

import inputSchema.InputSchemaFactory;
import inputSchema.Logo;
import inputSchema.Webproject;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Model;
import org.eclipse.uml2.uml.Stereotype;

import com.ibm.xtools.transform.authoring.RuleExtension;

public class Model2WebprojectRule implements RuleExtension {

	@SuppressWarnings("unchecked") //$NON-NLS-1$
	public void execute(EObject arg0, EObject arg1) {
		ModelUtil.log("Model2WebprojectRule", arg0); //$NON-NLS-1$
		Model model = (Model) arg0;
		Webproject proj = (Webproject) arg1;
		
		// Marker management
        IResource rsrc = ModelUtil.getModelResource(model);
        try {
        	if (rsrc != null) rsrc.deleteMarkers(IMarker.PROBLEM, true, IResource.DEPTH_INFINITE);
		} catch (CoreException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		// appName and logo are stored in <<settings>> classes
		proj.setAppName(model.getName()); // default
		List<Class> projClasses = new ArrayList<Class>();
		ModelUtil.extractClasses(model, Messages.settingsStereotype, projClasses);
		for (Class cl:projClasses) {
			Stereotype st = cl.getAppliedStereotype(Messages.settingsStereotype);
			if (st != null) {
				// Webproject appName
				String s = (String) cl.getValue(st, Messages.appnameProperty); //$NON-NLS-1$
				if (s != null && !s.equals("")) proj.setAppName(s); //$NON-NLS-1$
				// Webproject logo
	            String logoFName = (String) cl.getValue(st, Messages.logoFile);
	            if (logoFName != null && !logoFName.equals("")) { //$NON-NLS-1$
	                IProject prj = ModelUtil.getProject(model);
	                IFile f = prj.getFile(logoFName);
	                if (!f.exists()) {
	                    ModelUtil.addMarker(model, "Invalid logo file name " + logoFName //$NON-NLS-1$
	                             + ". The logo file should be located in the same project as the model itself. Examples: '" //$NON-NLS-1$
	                             + "logofile.gif' or 'subdir\\logofile.jpg'.", IMarker.SEVERITY_ERROR); //$NON-NLS-1$
	                } else {
	                    Logo logo = InputSchemaFactory.eINSTANCE.createLogo();
	                    proj.getLogo().add(logo);
	                    logo.setFileName(ModelUtil.getSimpleName(f.getFullPath().toOSString(), File.separatorChar));
	                    logo.setFilePath(ModelUtil.getPath(f.getFullPath().toOSString(), File.separatorChar).substring(1));
	                    Integer temp = (Integer) cl.getValue(st, Messages.logoWidth);
	                    if (temp != null) logo.setWidthInPixels(temp.toString());
	                    temp = (Integer) cl.getValue(st, Messages.logoHeight);
	                    if (temp != null) logo.setHeightInPixels(temp.toString());
	                }
	            }
			}
		}
	}
}
