package com.ibm.jps.uxtoweb.frontend.custom;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Operation;
import org.eclipse.uml2.uml.Property;

import com.ibm.xtools.transform.authoring.ExtractorExtension;

public class Operation2ForwardExtractor implements ExtractorExtension {

	public Collection execute(EObject arg0) {
		ModelUtil.log("Operation2ForwardExtractor", arg0);
		Operation inOp = (Operation) arg0;
		List<Property> c = new ArrayList<Property>();
		// The input operation is of type "user action", typically a button the user can click on. Using this 
		// action will normally cause the the application to navigate a new screen. The screen may be dependent
		// on the outcome of the user action (e.g. success for a logon action). 
		//
		// The target screens are identified as unidirectional associations from the parent class (a <<screen>>  
		// class) of the operation to other <<screen>> classes. For an operation 'addCd', the associations may be 
		// named as follows:
		// - association name = 'addCd' (equivalent to an unconditional local forward)
		// - association name = 'addCd [condition]' (equivalent to a conditional local forward)
		// - association name = '[condition]' (equivalent to a conditional global forward)
		// 
		// +---------+                                       +------------+
		// | Class1  |             addCd                     | <<screen>> |  
		// |---------|-------------------------------------->|   Class2   |
		// | addCd() |                                class2 |            |
		// +---------+                                       +------------+
		//
		// This extractor will return a collection of properties corresponding to the roles of the valid
		// associations. In the example above, the collection will contain only one property: 'class2'.
		
		// Retrieve all the candidate associations based on the operation name 
		Class parent = (Class) inOp.getOwner();
		List associations = ModelUtil.findAssociations(parent, inOp.getName());
		
		// Cycle though the associations; 
		for (Object acl_o:associations) {
			AssociatedClass acl = (AssociatedClass) acl_o;
			Class targetClass = acl.getAssocClass();
//			String assocName = acl.getAssoc().getName();
//			String forwardName = "success";
//			int pos1 = assocName.indexOf('[');
//			if (pos1 > 0) {
//				int pos2 = assocName.indexOf(']');
//				if (pos2 > 0) 
//					forwardName = assocName.substring(pos1 + 1, pos2);
//			}
			// The target class must be a screen
			if (targetClass.getAppliedStereotype(Messages.screenStereotype) != null) {
				c.add(acl.getRole());
			}
		}		
		return c;
	}

}
