package com.ibm.jps.uxtoweb.frontend.custom;

import org.eclipse.core.resources.IMarker;
import org.eclipse.emf.query.conditions.Condition;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Dependency;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.Namespace;
import org.eclipse.uml2.uml.Property;

public class OwnedAttribute2ItemCondition extends Condition {

	@Override
	public boolean isSatisfied(Object object) {
		ModelUtil.log("OwnedAttribute2ItemCondition", object); //$NON-NLS-1$
		Property inProp = (Property) object;
		Namespace ns = inProp.getNamespace();
		if (ns == null || !(ns instanceof Class)) return false;
		
		// This condition is used when identifying which fields of a <<list>> class should be actually 
		// shown in the list.
		
		// Condition #1: the property must be stereotyped <<display>>
		if (inProp.getAppliedStereotype("WebModeling::display") == null) return false;
		
		// Condition #2: the property to display must have an equivalent in an associated bean. 

		// The first part for this validation is to identified the bean associated with the list through a dependency. 
		// Note that, in the current version, a list can be associated with only one bean: therefore we stop at the first
		// valid bean (its name is stored in 'lastParentBean').
		// TODO Support multiple beans for list classes
		
		Class cl = (Class) ns;
		String lastParentBean = ""; //$NON-NLS-1$
		for (Object o:cl.getClientDependencies()){
			Dependency dep = (Dependency) o;
			for (Object p:dep.getSuppliers()) {
				NamedElement elt = (NamedElement) p;
				if (elt instanceof Class) {
					Class supplier = (Class) elt;
					if (supplier.getAppliedStereotype(Messages.beanStereotype) != null) {
						lastParentBean = supplier.getName();
						break;
					}
				}
			}
			if (!lastParentBean.equals("")) break; //$NON-NLS-1$
		}
		
		// The second part of the validation consists in retrieving the bean associated with our list ('cl') that has 
		// a matching property. If this bean doesn't match 'lastParentBean', then the attribute is ignored and a marker 
		// is added. Of course, a warning is also raised if the bean cannot be identified at all. 
		
		Class bean = ModelUtil.findMatchingBean(cl, inProp.getName());
		if (bean != null) {
			String parentBean = bean.getName();
			// Bean must be the same as the class
			if (!lastParentBean.equals("") && !parentBean.equals(lastParentBean)) { //$NON-NLS-1$
                ModelUtil.addMarker(cl, "The list attribute " + inProp.getName() + " (from " + cl.getName() + //$NON-NLS-1$ //$NON-NLS-2$
                		") points to the bean " + parentBean + " that is different from the bean " + //$NON-NLS-1$ //$NON-NLS-2$
                		lastParentBean + " of other attributes. This attribute has been ignored.", IMarker.SEVERITY_WARNING);							 //$NON-NLS-1$
			} else {
				return true;
			}
		} else {
            ModelUtil.addMarker(cl, "The list attribute " + inProp.getName() + " (from " + cl.getName() + //$NON-NLS-1$ //$NON-NLS-2$
            		") does not have a matching attribute in a supplier bean. This attribute has been ignored.",  //$NON-NLS-1$
            		IMarker.SEVERITY_WARNING);							
		}
		return false;
	}

}
