package com.ibm.jps.uxtoweb.frontend.custom;

import java.util.Collection;

import org.eclipse.core.resources.IMarker;
import org.eclipse.emf.common.util.BasicEList;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.ConnectableElement;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.Stereotype;

import com.ibm.xtools.transform.authoring.ExtractorExtension;

public class OwnedElement2InputFieldExtractor implements ExtractorExtension {

	@SuppressWarnings("unchecked") //$NON-NLS-1$
	public Collection execute(EObject arg0) {
		ModelUtil.log("OwnedElement2InputFieldExtractor", arg0);
		Class inCl = (Class) arg0;
		Collection<Property> c = new BasicEList();
		for (Object curElt_o:inCl.getRoles()) {
			ConnectableElement curElt = (ConnectableElement) curElt_o;
			// A <<screen>> may point to an <<input>> form using a composition. Note that we access the <<imput>> 
			// form though the role (property) of the navigable composition. 
			if (curElt instanceof Property) {
				Class cl = ModelUtil.findNavigableCompositionEnd((Property) curElt);
				if (cl != null && cl.getAppliedStereotype(Messages.inputStereotype) != null) {
					// We have identified an <<input>> class: cycle through its attributes. Candidate attributes have
					// specific stereotypes (like <<inputtext>>).
					String previousBean = ""; //$NON-NLS-1$
					for (Object curProp_o:cl.getAllAttributes()) {
						Property curProp = (Property) curProp_o;
						for (Object curSt_o:curProp.getAppliedStereotypes()) {
							Stereotype curSt = (Stereotype) curSt_o;
							String curStName = curSt.getQualifiedName();
							boolean valid = false;
							if (curStName.equals(Messages.inputtextStereotype) || 
								curStName.equals(Messages.listboxStereotype) || 
								curStName.equals(Messages.radiobuttonStereotype)) {
								// We have a candidate attribute. First need to check that it is valid:
								// - all candidate attributes must have a match in a bean and this must be the same
								//   bean for all attributes
								Class bean = ModelUtil.findMatchingBean(cl, curProp.getName());
								if (bean != null) {
									String beanName = bean.getName();
									if (!previousBean.equals("") && !beanName.equals(previousBean)) { //$NON-NLS-1$
						                ModelUtil.addMarker(cl, "The form attribute " + curProp.getName() + " (from " + cl.getName() + //$NON-NLS-1$ //$NON-NLS-2$
						                		") points to the bean " + beanName + " that is different from the bean " + //$NON-NLS-1$ //$NON-NLS-2$
						                		previousBean + " of other attributes. This attribute has been ignored.", IMarker.SEVERITY_WARNING);							 //$NON-NLS-1$
									} else {
										valid = true;
									}
								} else {
						            ModelUtil.addMarker(cl, "The form attribute " + curProp.getName() + " (from " + cl.getName() + //$NON-NLS-1$ //$NON-NLS-2$
						            		") does not have a matching attribute in a supplier bean. This attribute has been ignored.",  //$NON-NLS-1$
						            		IMarker.SEVERITY_WARNING);							
								}
								// Check that any sortable field is of type String
								if (valid) {
									Stereotype st = curProp.getAppliedStereotype(Messages.optionsStereotype);
									if (st != null) {
										boolean b = ((Boolean) curProp.getValue(st, Messages.sortableProperty)).booleanValue();
										if (b) {
											if (curProp.getType() == null || !(curProp.getType().getName().equals(Messages.stringLabel))) {
												valid = false;
								                ModelUtil.addMarker(curProp, "In the current version, sortable elements must be of type String. " + 
								                		"The 'sortable' option for attribute " + curProp.getName() + 
								                		" (in " + curProp.getNamespace().getName() + ") will be ignored.",
														IMarker.SEVERITY_WARNING);
											}
										}
									}
								}
								if (valid) {
									c.add(curProp);
								}
							}
						}
					}
				}
			}		
		}
		return c;
	}

}
