package com.ibm.jps.uxtoweb.frontend.custom;

import inputSchema.Field;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Dependency;
import org.eclipse.uml2.uml.Enumeration;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.Namespace;
import org.eclipse.uml2.uml.Property;
import org.eclipse.uml2.uml.Type;

import com.ibm.xtools.transform.authoring.RuleExtension;

public class Property2FieldRule implements RuleExtension {

	public void execute(EObject arg0, EObject arg1) {
		ModelUtil.log("Property2FieldRule", arg0);
		Property prop = (Property) arg0;
		Field field = (Field) arg1;
		// Field name
		field.setName(prop.getName());
		// Field columnName: cycle through the dependencies of the property. If the other end
		// (supplier") is a property belonging to a <<table>> class, then it is the matching column.
		for (Object o:prop.getClientDependencies()) {
			Dependency dep = (Dependency) o;
			for (Object o1:dep.getSuppliers()) {
				NamedElement elt = (NamedElement) o1;
				if (elt instanceof Property) {
					Property att = (Property) elt;
					Namespace ns = att.getNamespace();
					if (ns != null && ns instanceof Class) {
						Class cl = (Class) ns;
						if (cl.getAppliedStereotype(Messages.tableStereotype) != null) {
							field.setColumnName(att.getName());
						}
					}
				}
			}
		}
		// isPrimaryKey property
		field.setIsPrimaryKey(Messages.falseMark);
		if (prop.getAppliedStereotype(Messages.primaryKeyStereotype) != null) field.setIsPrimaryKey(Messages.trueMark);
		// Field type - note that there is an input filter that guarantees the type is defined
		Type t = prop.getType();
		if (t instanceof Enumeration) {
			field.setType(Messages.stringLabel);
		} else {
			field.setType(prop.getType().getName());
		}
		// isDerived
		// TODO inline should be an attribute of isDerived
		field.setIsDerived(Messages.falseMark);
		if (prop.isDerived()) field.setIsDerived(Messages.trueMark);
		// Inline - this is the doc of the property
		String doc = ModelUtil.getDocumentation(prop);
		if (!doc.equals("")) field.setInline(doc); //$NON-NLS-1$
	}
}
